
/*
 * Copyright (c) 2000 Blue Mug, Inc.  All Rights Reserved.
 */

#include <ceiva/ioregs.h>

#include "lcd.h"
#include "tags.h"

/* LCD control constants */
#define VIDEO_BUFSIZE ((SCREENWIDTH * SCREENHEIGHT * COLORDEPTH) / 128 - 1)
#define VIDEO_LINELENGTH	(SCREENWIDTH / 16 - 1)
#define PD1_LCDEN       	(1<<1)
#define PD5_LCDBL               (1<<5)


#define LCDRevisionCode		        0x00
#define LCDMemoryConf		        0x01
#define LCDPanelType		        0x02
#define LCDModRate		        0x03
#define LCDHorzDispWidth		0x04
#define LCDHorzNonDispWidth	        0x05
#define LCDLineStartPos		        0x06
#define LCDLinePulseWidth		0x07
#define LCDVertDispHeight0	        0x08
#define LCDVertDispHeight1	        0x09
#define LCDVertNonDispHeight	        0x0A
#define LCDFrameStartPos		0x0B
#define LCDFramePulseWidth	        0x0C
#define LCDDispMode		        0x0D
#define LCDScreen1LineComp0	        0x0E
#define LCDScreen1LineComp1	        0x0F
#define LCDScreen1DispStartAddr0	0x10
#define LCDScreen1DispStartAddr1	0x11
#define LCDScreen1DispStartAddr2	0x12
#define LCDScreen2DispStartAddr0	0x13
#define LCDScreen2DispStartAddr1	0x14
#define LCDScreen2DispStartAddr2	0x15
#define LCDMemAddrOffset0		0x16
#define LCDMemAddrOffset1		0x17
#define LCDPixelPan		        0x18
#define LCDClockConf		        0x19
#define LCDPowerSaveConf		0x1A
#define LCDMiscDisable		        0x1B
#define LCDMDConfRead0		        0x1C
#define LCDMDConfRead1		        0x1D
#define LCDGenIOConf0		        0x1E
#define LCDGenIOConf1		        0x1F
#define LCDGenIOControl0		0x20
#define LCDGenIOControl1		0x21
#define LCDPerfEnhance0		        0x22
#define LCDPerfEnhance1		        0x23
#define LCDLUTAddr		        0x24
#define LCDLUTData		        0x26
#define LCDCursorControl		0x27
#define LCDCursorX0		        0x28
#define LCDCursorX1		        0x29
#define LCDCursorY0		        0x2A
#define LCDCursorY1		        0x2B
#define LCDCursorColor00		0x2C
#define LCDCursorColor01		0x2D
#define LCDCursorColor10		0x2E
#define LCDCursorColor11		0x2F
#define LCDCursorStartAddr	        0x30
#define LCDAltFRM			0x31

static inline void lcd_writereg(uint8_t data, int reg) 
{
  *(uint8_t*)(VIDEO_REG_BASE+reg) = data;
}

static inline uint8_t lcd_readreg(int reg)
{
  return *(uint8_t*)(VIDEO_REG_BASE+reg);
}

static void lcd_clear(void)
{
	char *p = (char*) VIDEO_MEM_BASE;
	int i = VIDEO_MEM_SIZE;
	while (i--) *p++ = 0;
}

void lcd_init(void)
{
  unsigned int rev;
  unsigned int i;
  hprintf("LCD init...\n");
	/* Turn on controller + backlight */
	/*IO_PDDR |= (PD1_LCDEN | PD5_LCDBL);*/
        
        /*IO_MEMCFG1 = (IO_MEMCFG1 & 0xFF00FFFF) | 0x00B80000;*/
        hprintf("MEMCFG1: 0x%x\n",IO_MEMCFG1);
	/*hprintf("SYSFLG1: 0x%x\n",IO_SYSFLG1);*/
        IO_SYSCON1 = 0x401C0;
	IO_PDDDR = 0x10;
	IO_PDDR = 0x23;
	IO_PMPCON = 0x800;

	/* configure LCD control registers */
	lcd_writereg(0x00, LCDMiscDisable);
	rev = lcd_readreg(LCDRevisionCode);
	hprintf("LCD code:%x, %x, %x, %x, %x\n",rev,IO_SYSCON1,IO_PDDDR,IO_PDDR, IO_PMPCON);
	lcd_writereg(0xA0, LCDPerfEnhance1);
	lcd_writereg(0x30, LCDMemoryConf);
	lcd_writereg(0x24, LCDPerfEnhance0);
	lcd_writereg(0x26, LCDPanelType);
	lcd_writereg(0x00, LCDModRate);
	lcd_writereg(0x4F, LCDHorzDispWidth);
	lcd_writereg(0x03, LCDHorzNonDispWidth);
	lcd_writereg(0x00, LCDLineStartPos);
	lcd_writereg(0x00, LCDLinePulseWidth);
	lcd_writereg(0xEF, LCDVertDispHeight0);
	lcd_writereg(0x00, LCDVertDispHeight1);
	lcd_writereg(0x02, LCDVertNonDispHeight);
	lcd_writereg(0x00, LCDFrameStartPos);
	lcd_writereg(0x00, LCDFramePulseWidth);
	lcd_writereg(0x14, LCDDispMode);
	lcd_writereg(0xFF, LCDScreen1LineComp0);
	lcd_writereg(0x03, LCDScreen1LineComp1);
	lcd_writereg(0x00, LCDScreen1DispStartAddr0);
	lcd_writereg(0x00, LCDScreen1DispStartAddr1);
	lcd_writereg(0x00, LCDScreen1DispStartAddr2);
	lcd_writereg(0x00, LCDScreen2DispStartAddr0);
	lcd_writereg(0x00, LCDScreen2DispStartAddr1);
	lcd_writereg(0x00, LCDScreen2DispStartAddr2);
	lcd_writereg(0x80, LCDMemAddrOffset0);
	lcd_writereg(0x02, LCDMemAddrOffset1);
	lcd_writereg(0x00, LCDPixelPan);
	lcd_writereg(0x02, LCDClockConf);
	lcd_writereg(0x00, LCDPowerSaveConf);
	lcd_writereg(0x00, LCDMDConfRead0);
	lcd_writereg(0x00, LCDMDConfRead1);
	lcd_writereg(0x0E, LCDGenIOConf0);
	lcd_writereg(0x00, LCDGenIOConf1);
	lcd_writereg(0xFF, LCDGenIOControl0);
	lcd_writereg(0xFF, LCDGenIOControl1);
	lcd_writereg(0x00, LCDAltFRM);
	lcd_writereg(0x20, LCDPerfEnhance1);
	lcd_writereg(0x15, LCDDispMode);


	
	/* clear screen buffer */
	lcd_clear();
}

void lcd_fini(void)
{
	/* clear screen buffer */
	lcd_clear();

	/* Turn off controller + backlight */
	/*IO_PDDR &= ~(PD1_LCDEN | PD5_LCDBL);*/
}
