/*
 * Copyright (c) 2000 Blue Mug, Inc.  All Rights Reserved.
 */

#include <ep7211/ioregs.h>
#include <target/herrno.h>
#include <target/io.h>
#include "gunzip.h"
#include "linux.h"
#include "memmap.h"
#include "tags.h"
#include "memzero.h"
#include "lcd.h"

/* for setting the root device */
#define MINORBITS       8
#define MKDEV(ma,mi)    (((ma) << MINORBITS) | (mi))

#define RAMDISK_MAJOR   1	/* major for "RAM disk" */
#define RAMDISK0_MINOR	0	/* b 1 0 == /dev/ram0 */

#define MTD_BLOCK_MAJOR	31	/* major for "ROM/flash memory card" */
#define MTD0_MINOR_RO	0	/* b 31 16 == /dev/flash0 */
#define MTD1_MINOR_RO	1	/* b 31 17 == /dev/flash1 */

#define COMMAND_LINE_SIZE 1024

/* this comes straight from the kernel source */
struct param_struct {
	union {
		struct {
	 		unsigned long page_size;		/*  0 */
			unsigned long nr_pages;			/*  4 */
			unsigned long ramdisk_size;		/*  8 */
			unsigned long flags;			/* 12 */
#define FLAG_READONLY	1
#define FLAG_RDLOAD	4
#define FLAG_RDPROMPT	8
			unsigned long rootdev;			/* 16 */
			unsigned long video_num_cols;		/* 20 */
			unsigned long video_num_rows;		/* 24 */
			unsigned long video_x;			/* 28 */
			unsigned long video_y;			/* 32 */
			unsigned long memc_control_reg;		/* 36 */
			unsigned char sounddefault;		/* 40 */
			unsigned char adfsdrives;		/* 41 */
			unsigned char bytes_per_char_h;		/* 42 */
			unsigned char bytes_per_char_v;		/* 43 */
			unsigned long pages_in_bank[4];		/* 44 */
			unsigned long pages_in_vram;		/* 60 */
			unsigned long initrd_start;		/* 64 */
			unsigned long initrd_size;		/* 68 */
			unsigned long rd_start;			/* 72 */
			unsigned long system_rev;		/* 76 */
			unsigned long system_serial_low;	/* 80 */
			unsigned long system_serial_high;	/* 84 */
			unsigned long mem_fclk_21285;		/* 88 */
		} s;
		char unused[256];
	} u1;
	union {
		char paths[8][128];
		struct {
			unsigned long magic;
			char n[1024 - sizeof(unsigned long)];
		} s;
	} u2;
	char commandline[COMMAND_LINE_SIZE];
};

void start_linux(void)
{
	char *argv [2] = { "unzip", 0 };

#ifdef HAVE_LCD
	/* display splash screen */
	gunzip_splash();
#endif

	/* unzip and start kernel */
	(*unzip_command.func)((sizeof argv)/(sizeof *argv) - 1, argv);
	argv[0] = "linux";
	(*linux_command.func)((sizeof argv)/(sizeof *argv) - 1, argv);
}

static int linux_cmdfunc(int argc, char *argv[])
{
	struct tag *tag = (struct tag *) LINUX_PARAM_ADDRESS;

        lcd_init();

	/* zero param block */
	memzero (tag, sizeof(struct tag));

	/* set up core tag */
	tag->hdr.tag = ATAG_CORE;
	tag->hdr.size = tag_size(tag_core);
	tag->u.core.flags = 0;
	tag->u.core.pagesize = 0x1000;
	tag->u.core.rootdev = MKDEV(MTD_BLOCK_MAJOR, MTD0_MINOR_RO);

	/* 16 MB of SDRAM at 0xc0000000 */
	tag = tag_next(tag);
	tag->hdr.tag = ATAG_MEM;
	tag->hdr.size = tag_size(tag_mem32);
	tag->u.mem.size = DRAM1_SIZE >> 12;
	tag->u.mem.start = DRAM1_START;

	/* an initial ramdisk image in flash at 0x00700000 */
	/*tag = tag_next(tag);
	tag->hdr.tag = ATAG_INITRD;
	tag->hdr.size = tag_size(tag_initrd);
	tag->u.initrd.start = INITRD_LOAD_ADDRESS;
	tag->u.initrd.size  = INITRD_SIZE;*/

	/* the command line arguments */
	/*if (argc > 1) {*/
	tag = tag_next(tag);
	tag->hdr.tag = ATAG_CMDLINE;
	tag->hdr.size = (COMMAND_LINE_SIZE + 3 +
			 sizeof(struct tag_header)) >> 2;
	
	{
	         const unsigned char *src;
		 unsigned char *dst;
		 dst = tag->u.cmdline.cmdline;
		 memzero (dst, COMMAND_LINE_SIZE);
		 *dst++='m';
		 *dst++='e';
		 *dst++='m';
		 *dst++='=';
		 *dst++='4';
		 *dst++='M';
		 *dst++=' ';
		 while (--argc > 0) {
		         src = *++argv;
			 hprintf ("Doing %s\n", src);
			 while (*src)
			   *dst++ = *src++;
			 *dst++ = ' ';
		 }
		 
		 *--dst = '\0';
		 hprintf("Command line: '%s'\n",tag->u.cmdline.cmdline);
	}
	/*}*/

	tag = tag_next(tag);
	tag->hdr.tag = 0;
	tag->hdr.size = 0;


	/* branch to kernel image */
	__asm__ volatile (
	"	mov	r4, #0xC0000000\n"	/* start of DRAM */
	"	add	r4, r4, #0x00028000\n"	/* kernel offset */
	"	mov	r0, #0\n"		/* kernel sanity check */
	"	mov	r1, #50\n"		/* ceiva arch. number */
	"	mov	r2, #0\n"
	"	mov	r3, #0\n"
	"	mov	pc, r4"			/* go there! */
	);

	/* never get here */
	return 0;
}

static int unzip_cmdfunc(int argc, char *argv[])
{
	/* turn on LED flasher (25% duty cycle) */
	IO_LEDFLSH = 0x4c; 
	
	/* decompress kernel image to DRAM */
	gunzip_object(" kernel",LINUX_FLASH_ADDRESS, LINUX_LOAD_ADDRESS);
	//gunzip_object("ramdisk",ramdisk, INITRD_LOAD_ADDRESS);

	/* turn off LED flasher */
	IO_LEDFLSH = 0;

	return 0;
}

static int boot_cmdfunc(int argc, char *argv[])
{
	if (argc < 1)
		return -H_EUSAGE;
	
	IO_LEDFLSH = 0x4c; 
	gunzip_object (" kernel", 0x00010000, LINUX_LOAD_ADDRESS);
	gunzip_object ("ramdisk", 0x10000000, INITRD_LOAD_ADDRESS);
	IO_LEDFLSH = 0;

	linux_cmdfunc (argc, argv);

	return 0;
}


const command_t linux_command =
	{ "linux", "<linux options>", "start Linux", &linux_cmdfunc };
const command_t unzip_command =
	{ "unzip", 0, "unzip Linux kernel", &unzip_cmdfunc };
const command_t boot_command =
	{ "boot", "", "boot default Linux kernel and ramdisk", &boot_cmdfunc };
